
package com.tmerclub.cloud.flow.controller.multishop;

import com.tmerclub.cloud.api.order.vo.FormExcelParamVO;
import com.tmerclub.cloud.common.database.dto.PageDTO;
import com.tmerclub.cloud.common.database.vo.PageVO;
import com.tmerclub.cloud.common.exception.LuckException;
import com.tmerclub.cloud.common.i18n.I18nMessage;
import com.tmerclub.cloud.common.response.ServerResponseEntity;
import com.tmerclub.cloud.common.security.AuthUserContext;
import com.tmerclub.cloud.common.util.BeanUtil;
import com.tmerclub.cloud.common.util.ExcelUtil;
import com.tmerclub.cloud.flow.dto.FormDTO;
import com.tmerclub.cloud.flow.model.Form;
import com.tmerclub.cloud.flow.service.FormService;
import com.tmerclub.cloud.flow.vo.FormExcelVO;
import com.tmerclub.cloud.flow.vo.FormItemVO;
import com.tmerclub.cloud.flow.vo.FormVO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * 数据表格
 *
 * @author TRACK
 */
@RestController("multishopFormController")
@RequestMapping("/m/form")
@Tag(name = "数据表格")
public class FormController {

    @Autowired
    private FormService formService;


    @GetMapping("/page")
    @Operation(summary = "获取数据表格列表", description = "分页获取数据表格列表")
    public ServerResponseEntity<PageVO<Form>> page(@Valid PageDTO pageDTO, FormDTO formParam) {
        Long shopId = AuthUserContext.get().getTenantId();
        formParam.setShopId(shopId);
        PageVO<Form> formPage = formService.page(pageDTO, formParam);
        return ServerResponseEntity.success(formPage);
    }

    @GetMapping
    @Operation(summary = "获取数据表格", description = "根据formId获取数据表格")
    public ServerResponseEntity<Form> getByFormId(@RequestParam Long formId) {
        Form form = formService.getByFormId(formId);
        if (Objects.isNull(form)) {
            throw new LuckException("当前报表不存在，请刷新页面重试");
        }
        return ServerResponseEntity.success(form);
    }

    @PostMapping
    @Operation(summary = "保存数据表格", description = "保存数据表格")
    public ServerResponseEntity<Void> save(@Valid @RequestBody FormDTO formDTO) {
        Form form = BeanUtil.map(formDTO, Form.class);
        form.setShopId(AuthUserContext.get().getTenantId());
        form.setFormId(null);
        formService.save(form);
        return ServerResponseEntity.success();
    }

    @PutMapping
    @Operation(summary = "更新数据表格", description = "更新数据表格")
    public ServerResponseEntity<Void> update(@Valid @RequestBody FormDTO formDTO) {
        Form form = BeanUtil.map(formDTO, Form.class);
        Long shopId = AuthUserContext.get().getTenantId();
        Form byFormId = formService.getByFormId(form.getFormId());
        if (!Objects.equals(byFormId.getShopId(), shopId)) {
            throw new LuckException("您没有权限更改该报表的数据");
        }
        formService.update(form);
        return ServerResponseEntity.success();
    }

    @DeleteMapping
    @Operation(summary = "删除数据表格", description = "根据数据表格id删除数据表格")
    public ServerResponseEntity<Void> delete(@RequestParam Long formId) {
        formService.deleteById(formId);
        return ServerResponseEntity.success();
    }

    @GetMapping("/get_form_item")
    @Operation(summary = "获取报表项列表", description = "获取报表项列表")
    public ServerResponseEntity<List<FormItemVO>> getFormItem() {
        List<FormItemVO> formItemEnumList = formService.getFormItem(2, I18nMessage.getLang());
        return ServerResponseEntity.success(formItemEnumList);
    }

    @GetMapping("/form_excel")
    @Operation(summary = "根据报表id生成对应的报表统计数据", description = "根据报表id生成对应的报表统计数据")
    public void formExcel(@RequestParam("formId") Long formId, HttpServletResponse response) {
        Form form = formService.getByFormId(formId);
        if (Objects.isNull(form)) {
            throw new LuckException("商家报表不存在");
        }
        FormExcelParamVO excelParam = formService.getExcelParam(form);
        List<FormExcelVO> list = formService.listExcel(form, excelParam);
        ExcelUtil.soleExcel(response, list, form.getFormName(), FormExcelVO.MERGE_ROW_INDEX, FormExcelVO.MERGE_COLUMN_INDEX, excelParam.getFormItemNames(), FormExcelVO.class);
    }

    @GetMapping("/get_recommend_form_list")
    @Operation(summary = "获取全部推荐报表", description = "获取全部推荐报表")
    public ServerResponseEntity<List<FormVO>> getRecommendFormList() {
        List<FormVO> formList = formService.getRecommendFormList();
        return ServerResponseEntity.success(formList);
    }
}
